/* pggimpl.h - macros used by the object implementations
 *      Copyright (C) 1999 Michael Roth <mroth@gnupg.org>
 *
 * This file is part of PGG (Privacy Guard Glue).
 *
 * PGG is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * PGG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */


#ifndef PGG_INCLUDE_PGGIMPL_H_19990616130037
#define PGG_INCLUDE_PGGIMPL_H_19990616130037


#define _ptr(type, arg)		((type##Ptr)(arg))
#define _hd(type, arg)		((type)(arg))
#define _malloc(type)		((type##Ptr)malloc(_size(type)))
#define _size(type)		sizeof(_mktype(type))
#define _mktype(type)		struct _##type
#define _mktypeptr(type)	typedef _mktype(type) * type##Ptr


#endif


